import { FileExtractorObject } from './FileExtractorObject';
import { FileMoveObject } from './FileMoveObject';
/**
 * Used for the extraction of linked files in the HTML.
 * This will be used to copy user linked files to the output directory and
 * update the HTML.
 */
export declare class FileExtractor {
    private static imageSrcRegExp;
    private static scriptSrcRegExp;
    private static linkHrefRegExp;
    private static videoSourceSrcRegExp;
    private static notHttpRegExp;
    /**
     * Copies all files given by their FileMoveObjects from their inputRoot
     * to the outputRoot.
     * @param files the files to copy
     * @param inputRoot the root for relative paths as input
     * @param outputRoot the root directory to copy the files to
     * @param timeout an optional timeout in ms.
     */
    static extractAll(files: FileMoveObject[], inputRoot: string, outputRoot: string, timeout?: number): Promise<void>;
    /**
     * Replace all links in the HTML. This will assert that the files
     * were extracted to a `assets` folder.
     * The returned `FileExtractorObject` will contain the updated HTML with
     * relative paths to the assets folder and a list of `FileMoveObject`s
     * used by `extractAll()` to actually extract the files.
     * @param html the html to change
     */
    static replaceAllLinks(html: string): FileExtractorObject;
    /**
     * Replace all image sources.
     * @param html The html to parse.
     */
    private static replaceImages;
    /**
     * Replace all script sources.
     * @param html The html to parse.
     */
    private static replaceScripts;
    /**
     * Replace all style sources.
     * @param html The html to parse.
     */
    private static replaceStyleSheets;
    /**
     * Replace all video sources.
     * @param html The html to parse.
     */
    private static replaceVideoSource;
    /**
     * Process the source replacement. Replacement function used for specific
     * regexp replacement.
     * @param wholeMatch the whole match
     * @param tag the html elements tag (e.g. `img`)
     * @param before all attributes before the actual attribute containing the path (e.g. `style="..."`)
     * @param wrapBefore the wrapping char (e.g. `"`) of the last attribute before the necessary [only used inside regex]
     * @param wrap the wrapping char (e.g. `"`) of the necessary element
     * @param val the actual path
     * @param after everything after the necessary attribute
     * @param closingSlash a closing slash if included
     * @param files the list of `FileMoveObject` to append this one to
     */
    private static processSourceReplacement;
    /**
     * Copy a file, creates nonexistent directories
     *
     * @return Promise<void>: resolves when done.
     */
    private static copyFile;
}
