/// <reference types="node" />
import fs from "fs";
import { ConversionObject } from "../objects/export/ConversionObject";
import { ConverterSettingsObject } from "../objects/settings/ConverterSettingsObject";
import { IConverter } from "./IConverter";
import { IShowdownConverter } from "./IShowdownConverter";
/**
 * Contains basic implementations used in the actual converters.
 */
export declare abstract class AConverter implements IConverter {
    /**
     * The converter used for the main body markdown conversion.
     */
    protected abstract converter: IShowdownConverter;
    abstract toHtml(markdown: string, options?: ConversionObject): Promise<string>;
    abstract toFile(markdown: string, file: string, rootPath: string, options?: ConversionObject, forceOverwrite?: boolean): Promise<string>;
    getOption(opt: string): any;
    setOption(opt: string, val: any): void;
    setOptions(options: ConverterSettingsObject): void;
    /**
     * Checks if the file can be opened in writing mode. Can be used to
     * check if a file is opened in another process before trying to overwrite
     * it.
     *
     * @param file a pathlike parameter declaring the file to check.
     *
     * @return a promise resolved if it can be opened.
     * Rejected with the error if not.
     */
    protected tryFileOpen(file: fs.PathLike): Promise<void>;
}
