"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const ConverterSettingsObject_1 = require("../objects/settings/ConverterSettingsObject");
/**
 * Contains basic implementations used in the actual converters.
 */
class AConverter {
    getOption(opt) {
        return this.converter.getOption(opt);
    }
    setOption(opt, val) {
        this.converter.setOption(opt, val);
    }
    setOptions(options) {
        let checkedObject = new ConverterSettingsObject_1.ConverterSettingsObject(options);
        let inputKeys = Object.keys(options);
        Object.keys(checkedObject).forEach((key) => {
            // always check if the option is in the input object
            // this way no other option keys are set to default again
            if (inputKeys.indexOf(key) >= 0)
                this.setOption(key, checkedObject[key]);
        });
    }
    /**
     * Checks if the file can be opened in writing mode. Can be used to
     * check if a file is opened in another process before trying to overwrite
     * it.
     *
     * @param file a pathlike parameter declaring the file to check.
     *
     * @return a promise resolved if it can be opened.
     * Rejected with the error if not.
     */
    tryFileOpen(file) {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((res, rej) => {
                fs_1.default.open(file, 'r+', (error, fd) => {
                    if (fd !== undefined) {
                        try {
                            fs_1.default.closeSync(fd);
                        }
                        catch (err) {
                            console.error(err);
                        }
                    }
                    if (!error || error.code === "ENOENT") {
                        res();
                    }
                    else {
                        rej(error);
                    }
                });
            });
        });
    }
}
exports.AConverter = AConverter;
