import Showdown from 'showdown';
/**
 * Creates Showdown extensions for elearnjs markdown conversion.
 */
export declare class ShowdownExtensionManager {
    private static readonly indentionRegExp;
    private static readonly inlineHeadingRegExp;
    private static readonly commentRegExp;
    private static readonly sectionRegExp;
    private static readonly firstSectionRegExp;
    private static readonly imprintRegExp;
    private static readonly metaBlockRegExp;
    private static readonly metaBlockElementsRegExp;
    private static readonly hideInOverviewRegExp;
    private static readonly noSectionRegExp;
    private static readonly secDescriptionRegExp;
    private static readonly secDescriptionReplacementRegExp;
    private static readonly hiddenCommentRegExp;
    private static readonly markdownDisabledRegExp;
    private static readonly markdownDisabledReplacementRegExp;
    private descriptions;
    private descriptionIndex;
    private disabledContents;
    private disabledContentsIndex;
    /**
     * Add the section syntax for new sections before headings.
     */
    private addSectionOnHeading;
    /**
     * Replace the section syntax with actual HTML.
     */
    private readonly replaceSectionSyntax;
    /**
     * Replace disabled markdown comments to be excluded.
     */
    private readonly replaceDisabledSyntax;
    /**
     * Insert stored disabled contents back into the code.
     */
    private readonly insertDisabledContents;
    /**
     * Replace the section syntax with actual HTML. Might add a page break
     * additionally if activated.
     */
    private readonly pdfSectionSyntax;
    /**
     * Insert stored descriptions back into the code.
     */
    private readonly insertSectionDescription;
    /**
     * Remove the meta block from the code.
     */
    private readonly removeMetaBlock;
    /**
     * Parse the imprints Markdown.
     */
    private readonly parseImprint;
    /**
     * Remove the imprint block from the code.
     */
    private readonly removeImprintBlock;
    /**
     * Remove the `no-section` comment.
     */
    private readonly cleanNoSectionComment;
    /**
     * Remove the `hide-in-overview` comment.
     */
    private readonly cleanHideInOverviewComment;
    /**
     * Remove the `desc` comment.
     */
    private readonly cleanSectionDescriptionComment;
    /**
     * Remove empty paragraphs from the output.
     */
    private readonly cleanEmptyParagraphs;
    /**
     * Remove the markdown="X" HTML attribute.
     */
    private readonly cleanMarkdownAttribute;
    /**
     * Remove hidden comments.
     */
    private readonly cleanHiddenComments;
    /**
     * Remove all HTML comments.
     */
    private readonly cleanHtmlComments;
    private static escapeSectionName;
    private static unescapeSectionName;
    private static escapeHTMLQuotes;
    private static removeMarkdownSyntax;
    /**
     * Removes up to @param indentionSize spaces (or tabs if tabsize <= indentionSize)
     * from the beginning of each line.
     */
    private static removeIndention;
    /**
     * Create a meta element based on the parameters.
     * @param tag the HTML tag.
     * @param value the value to set it to.
     * @param valueSurrounding the value surrounding (to be able to unescape)
     */
    private static createMeta;
    /**
     * Get all extensions used for the HTML body conversion.
     */
    getHtmlBodyExtensions(): (Showdown.FilterExtension | Showdown.ListenerExtension)[];
    /**
     * Get all extensions used for the PDF body conversion.
     */
    getPdfBodyExtensions(): (Showdown.FilterExtension | Showdown.ListenerExtension)[];
    /**
     * Get all extensions used for the imprint conversion.
     */
    getImprintExtensions(): Showdown.FilterExtension[];
    /**
     * Parse the meta data, to create all meta elements.
     * @param text the meta content.
     */
    parseMetaData(text: string): string;
    /**
     * Parse the code for section syntax.
     * @param converter the converter used to convert descriptions.
     * @param wholeMatch the whole section syntax (with additional comments if set)
     * @param wrap the wrapping char
     * @param heading the actual heading content
     * @param addition additional text, e.g. comments
     */
    private parseSection;
}
