"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AInheritingObject_1 = require("../AInheritingObject");
const InclusionObject_1 = require("./InclusionObject");
/**
 * An object containing information about included extensions, selected language
 * and additional conversion options, like `bodyOnly`,
 * `automaticExtensionDetection` and `removeComments`.
 *
 * Extending the `InclusionObject`
 */
class ConversionObject extends InclusionObject_1.InclusionObject {
    /**
     * An Object containing options for the general conversions
     * of the HtmlConverter and PdfConverter functions.
     *
     * @param {object} options: Manual values for this objects fields. All optional.
     */
    constructor(options) {
        super(options);
        /**
         * bodyOnly will only return the HTML body.
         * Default: false
         */
        this.bodyOnly = false;
        /**
         * automaticExtensionDetection will scan for extensions and
         * include only those found. Will only autodetect extensions with
         * undefined as `includeEXTENSION` value.
         * Only if not `bodyOnly`
         * Default: false
         */
        this.automaticExtensionDetection = false;
        /**
         * Whether to remove all HTML comments from the conversion output.
         * Default: false - no removal.
         */
        this.removeComments = false;
        AInheritingObject_1.AInheritingObject.inheritValues(this, ConversionObject.CLASS_KEYS, options);
    }
}
ConversionObject.CLASS_KEYS = [
    "bodyOnly",
    "automaticExtensionDetection",
    "removeComments",
];
exports.ConversionObject = ConversionObject;
