"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AInheritingObject_1 = require("../AInheritingObject");
/**
 * An object containing general converter settings. These apply for the
 * HtmlConverter and the PdfConverter but might be extended by specific objects.
 */
class ConverterSettingsObject extends AInheritingObject_1.AInheritingObject {
    /**
     * An Object containing options for the general Converters
     *
     * @param {object} options: Manual values for this objects fields. All optional.
     */
    constructor(options) {
        super(options);
        /**
         * Whether sections are automatically created at headings.
         * Default: true
         */
        this.newSectionOnHeading = true;
        /**
         * until which depth headings are created.
         * Default: 3 (H3)
         */
        this.headingDepth = 3;
        /**
         * Whether sub- and subsubsections are created
         * at a specific heading level (subSectionLevel, subsubSectionLevel)
         * Default: true.
         */
        this.useSubSections = true;
        /**
         * level from which on created sections are subsections.
         * Default: 3 (H3)
         */
        this.subSectionLevel = 3;
        /**
         * level from which on sections are subsubsections.
         * Default: 4 (H4) (will not be created with everything as default)
         */
        this.subsubSectionLevel = 4;
        AInheritingObject_1.AInheritingObject.inheritValues(this, ConverterSettingsObject.CLASS_KEYS, options);
    }
}
ConverterSettingsObject.CLASS_KEYS = [
    "newSectionOnHeading",
    "headingDepth",
    "useSubSections",
    "subSectionLevel",
    "subsubSectionLevel",
];
exports.ConverterSettingsObject = ConverterSettingsObject;
