import Puppeteer from 'puppeteer';
import { ConverterSettingsObject } from "./ConverterSettingsObject";
/**
 * An object extending the general `ConverterSettingsObject` by specific PDF
 * settings.
 */
export declare class PdfSettingsObject extends ConverterSettingsObject {
    protected static readonly CLASS_KEYS: string[];
    /**
     * will add page breaks before every section
     */
    newPageOnSection?: boolean;
    /**
     * zoom factor for the page rendering
     */
    contentZoom?: number;
    /**
     * HTML of a custom page header
     */
    customHeader?: string;
    /**
     * HTML of a custom page footer
     */
    customFooter?: string;
    /**
     * CSS declaration of the header's height
     */
    headerHeight?: string;
    /**
     * CSS declaration of the footer's height
     */
    footerHeight?: string;
    /**
     * absolute path to a styling css file
     */
    customStyleFile?: string;
    /**
     * absolute path to the chrome executable.
     * See https://github.com/GoogleChrome/puppeteer#default-runtime-settings
     */
    chromePath?: string;
    /**
     * Options used to launch puppeteer.
     * See https://pptr.dev/#?product=Puppeteer&version=v1.8.0&show=api-puppeteerlaunchoptions
     *
     * The `Puppeteer.LaunchOptions.executablePath` might be overwritten by
     * the option `chromePath` if set.
     */
    puppeteerOptions?: Puppeteer.LaunchOptions;
    /**
     * If set to true, the chrome browser process used in the `PdfConverter`
     * will be kept running until the option is set to false again.
     * Otherwise the browser will close automatically after conversion.
     */
    keepChromeAlive?: boolean;
    /**
     * An Object containing options for the PdfConverter
     *
     *  @param {PdfSettingsObject} options:
     *      Manual values for this objects fields. All optional.
     */
    constructor(options?: PdfSettingsObject);
}
