"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AInheritingObject_1 = require("../AInheritingObject");
const ConverterSettingsObject_1 = require("./ConverterSettingsObject");
/**
 * An object extending the general `ConverterSettingsObject` by specific PDF
 * settings.
 */
class PdfSettingsObject extends ConverterSettingsObject_1.ConverterSettingsObject {
    /**
     * An Object containing options for the PdfConverter
     *
     *  @param {PdfSettingsObject} options:
     *      Manual values for this objects fields. All optional.
     */
    constructor(options) {
        super(options);
        /**
         * will add page breaks before every section
         */
        this.newPageOnSection = true;
        /**
         * zoom factor for the page rendering
         */
        this.contentZoom = 1;
        /**
         * CSS declaration of the header's height
         */
        this.headerHeight = "0";
        /**
         * CSS declaration of the footer's height
         */
        this.footerHeight = "17mm";
        /**
         * If set to true, the chrome browser process used in the `PdfConverter`
         * will be kept running until the option is set to false again.
         * Otherwise the browser will close automatically after conversion.
         */
        this.keepChromeAlive = false;
        AInheritingObject_1.AInheritingObject.inheritValues(this, PdfSettingsObject.CLASS_KEYS, options);
    }
}
PdfSettingsObject.CLASS_KEYS = [
    "newPageOnSection",
    "contentZoom",
    "customHeader",
    "customFooter",
    "headerHeight",
    "footerHeight",
    "customStyleFile",
    "chromePath",
    "puppeteerOptions",
    "keepChromeAlive",
];
exports.PdfSettingsObject = PdfSettingsObject;
