"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const rimraf_1 = __importDefault(require("rimraf"));
const main_1 = require("../main");
const PromiseCounter_1 = require("../util/PromiseCounter");
const AssertExtensions_1 = __importDefault(require("./helpers/AssertExtensions"));
const pathToTestAssets = '../../testAssets/';
const pathToAssets = '../../assets/elearnjs/';
describe('Extension Manager', () => {
    afterEach(() => __awaiter(this, void 0, void 0, function* () {
        let promises = [];
        // remove all folders that might have been created
        let foldersToRemove = ["export"];
        foldersToRemove.forEach((folder) => {
            let promise = new Promise((res, rej) => {
                rimraf_1.default(path_1.default.join(__dirname, pathToTestAssets, folder), (err) => {
                    if (err)
                        rej(err);
                    else
                        res();
                });
            });
            promises.push(promise);
        });
        yield new PromiseCounter_1.PromiseCounter(promises, 15000);
    }));
    // basic body only test
    it('export assets without extension correctly', (done) => {
        // create output folder
        fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
        // export assets
        main_1.ExtensionManager.exportAssets(path_1.default.join(__dirname, pathToTestAssets, "export"), {}).then(() => {
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "elearn.js"), path_1.default.join(__dirname, pathToAssets, "assets", "js", "elearn.js"));
            assert_1.default.ok(!fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "quiz.js")));
            assert_1.default.ok(!fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "elearnvideo.js")));
            assert_1.default.ok(!fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "clickimage.js")));
            assert_1.default.ok(!fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "timeslider.js")));
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "font", "eLearn-Icons.woff"), path_1.default.join(__dirname, pathToAssets, "assets", "font", "eLearn-Icons.woff"));
            done();
        }, (err) => {
            done(err);
        });
    });
    // basic body only test
    it('exports selected assets correctly', () => __awaiter(this, void 0, void 0, function* () {
        // create output folder
        fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
        // export assets
        yield main_1.ExtensionManager.exportAssets(path_1.default.join(__dirname, pathToTestAssets, "export"), {
            includeQuiz: true,
            includeClickImage: true,
        });
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "elearn.js"), path_1.default.join(__dirname, pathToAssets, "assets", "js", "elearn.js"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "quiz.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "quiz", "assets", "js", "quiz.js"));
        assert_1.default.ok(!fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "elearnvideo.js")));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "clickimage.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "clickimage", "assets", "js", "clickimage.js"));
        assert_1.default.ok(!fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "timeslider.js")));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "font", "eLearn-Icons.woff"), path_1.default.join(__dirname, pathToAssets, "assets", "font", "eLearn-Icons.woff"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "lang", "elearnjs-en.js"), path_1.default.join(__dirname, pathToAssets, "assets", "lang", "elearnjs-en.js"));
    }));
    // basic body only test
    it('exports all assets correctly', () => __awaiter(this, void 0, void 0, function* () {
        // create output folder
        fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
        // export assets
        yield main_1.ExtensionManager.exportAssets(path_1.default.join(__dirname, pathToTestAssets, "export"), {
            includeQuiz: true,
            includeElearnVideo: true,
            includeClickImage: true,
            includeTimeSlider: true,
        });
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "elearn.js"), path_1.default.join(__dirname, pathToAssets, "assets", "js", "elearn.js"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "quiz.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "quiz", "assets", "js", "quiz.js"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "css", "quiz.css"), path_1.default.join(__dirname, pathToAssets, "extensions", "quiz", "assets", "css", "quiz.css"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "elearnvideo.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "elearnvideo", "assets", "js", "elearnvideo.js"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "css", "elearnvideo.css"), path_1.default.join(__dirname, pathToAssets, "extensions", "elearnvideo", "assets", "css", "elearnvideo.css"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "font", "eLearn-Video.ttf"), path_1.default.join(__dirname, pathToAssets, "extensions", "elearnvideo", "assets", "font", "eLearn-Video.ttf"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "font", "eLearn-Video.woff"), path_1.default.join(__dirname, pathToAssets, "extensions", "elearnvideo", "assets", "font", "eLearn-Video.woff"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "clickimage.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "clickimage", "assets", "js", "clickimage.js"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "css", "clickimage.css"), path_1.default.join(__dirname, pathToAssets, "extensions", "clickimage", "assets", "css", "clickimage.css"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "timeslider.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "timeslider", "assets", "js", "timeslider.js"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "css", "timeslider.css"), path_1.default.join(__dirname, pathToAssets, "extensions", "timeslider", "assets", "css", "timeslider.css"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "font", "eLearn-Icons.woff"), path_1.default.join(__dirname, pathToAssets, "assets", "font", "eLearn-Icons.woff"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "lang", "elearnjs-en.js"), path_1.default.join(__dirname, pathToAssets, "assets", "lang", "elearnjs-en.js"));
        AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "lang", "elearnjs-de.js"), path_1.default.join(__dirname, pathToAssets, "assets", "lang", "elearnjs-de.js"));
    }));
    it('detects no extensions in basic html', () => {
        let html = `<h1>This is basic HTML without any extension</h1>`;
        let extensionObject = main_1.ExtensionManager.scanHtmlForAll(html);
        assert_1.default.deepEqual(extensionObject, new main_1.ExtensionObject({
            includeClickImage: false,
            includeElearnVideo: false,
            includeQuiz: false,
            includeTimeSlider: false,
        }));
    });
    it('detects all extensions in markdown', (done) => {
        let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
        let data = inBuf.toString();
        main_1.ExtensionManager.scanMarkdownForAll(data).then((extensionObject) => {
            assert_1.default.deepEqual(extensionObject, new main_1.ExtensionObject({
                includeClickImage: true,
                includeElearnVideo: true,
                includeQuiz: true,
                includeTimeSlider: true,
            }));
            done();
        });
    });
    it('detects all extensions in markdown with given HtmlConverter', (done) => {
        let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
        let data = inBuf.toString();
        let htmlConverter = new main_1.HtmlConverter();
        main_1.ExtensionManager.scanMarkdownForAll(data, htmlConverter).then((extensionObject) => {
            assert_1.default.deepEqual(extensionObject, new main_1.ExtensionObject({
                includeClickImage: true,
                includeElearnVideo: true,
                includeQuiz: true,
                includeTimeSlider: true,
            }));
            done();
        });
    });
    it('detects all extensions in markdown with given PdfConverter', (done) => {
        let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
        let data = inBuf.toString();
        let pdfConverter = new main_1.PdfConverter();
        main_1.ExtensionManager.scanMarkdownForAll(data, pdfConverter).then((extensionObject) => {
            assert_1.default.deepEqual(extensionObject, new main_1.ExtensionObject({
                includeClickImage: true,
                includeElearnVideo: true,
                includeQuiz: true,
                includeTimeSlider: true,
            }));
            done();
        });
    });
});
