"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const rimraf_1 = __importDefault(require("rimraf"));
const FileMoveObject_1 = require("../FileMoveObject");
const main_1 = require("../main");
const PromiseCounter_1 = require("../util/PromiseCounter");
const pathToTestAssets = '../../testAssets/';
describe("FileExtractor", () => {
    afterEach(() => __awaiter(this, void 0, void 0, function* () {
        let promises = [];
        // remove all folders that might have been created
        let foldersToRemove = ["export"];
        foldersToRemove.forEach((folder) => {
            let promise = new Promise((res, rej) => {
                rimraf_1.default(path_1.default.join(__dirname, pathToTestAssets, folder), (err) => {
                    if (err)
                        rej(err);
                    else
                        res();
                });
            });
            promises.push(promise);
        });
        yield new PromiseCounter_1.PromiseCounter(promises, 15000);
    }));
    describe("Positive Tests", () => {
        it("extracts an absolute filepath correctly", (done) => {
            fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
            let inputPath = path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExample.pdf`);
            assert_1.default.ok(path_1.default.isAbsolute(inputPath));
            let objects = [];
            // copy to itself, should be ignored
            objects.push(new FileMoveObject_1.FileMoveObject(inputPath, path_1.default.join("export", `testTemplateExample.pdf`)));
            main_1.FileExtractor.extractAll(objects, __dirname, path_1.default.join(__dirname, pathToTestAssets), 15000).then(() => {
                try {
                    assert_1.default.ok(fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", `testTemplateExample.pdf`)));
                }
                catch (err) {
                    return done(err);
                }
                done();
            }, (err) => {
                done(err);
            });
        }).timeout(20000);
        it("extracts all types correctly from html", () => {
            let img = `<img src="img1"/>
                        <img style="" src="img2" alt="text"/>`;
            let source = `<source src="vid1"/>
                        <img style="" src="vid2" alt="text"/>`;
            let script = `<script src="scr1"></script>
                        <script style="" src="scr2" alt="text"></script>`;
            let link = `<link href="lnk1"/>
                        <link style="" href="lnk2" alt="text"/>`;
            let html = `${img}${source}${script}${link}`;
            // actual replacement
            let feo = main_1.FileExtractor.replaceAllLinks(html);
            let htmlExpected = html.replace(/img1/, "assets/img/img1")
                .replace(/img2/, "assets/img/img2")
                .replace(/vid1/, "assets/img/vid1")
                .replace(/vid2/, "assets/img/vid2")
                .replace(/scr1/, "assets/js/scr1")
                .replace(/scr2/, "assets/js/scr2")
                .replace(/lnk1/, "assets/css/lnk1")
                .replace(/lnk2/, "assets/css/lnk2");
            assert_1.default.equal(feo.html, htmlExpected);
            let filesExpected = [];
            filesExpected.push(new FileMoveObject_1.FileMoveObject("img1", "assets/img/img1"));
            filesExpected.push(new FileMoveObject_1.FileMoveObject("img2", "assets/img/img2"));
            filesExpected.push(new FileMoveObject_1.FileMoveObject("vid1", "assets/img/vid1"));
            filesExpected.push(new FileMoveObject_1.FileMoveObject("vid2", "assets/img/vid2"));
            filesExpected.push(new FileMoveObject_1.FileMoveObject("scr1", "assets/js/scr1"));
            filesExpected.push(new FileMoveObject_1.FileMoveObject("scr2", "assets/js/scr2"));
            filesExpected.push(new FileMoveObject_1.FileMoveObject("lnk1", "assets/css/lnk1"));
            filesExpected.push(new FileMoveObject_1.FileMoveObject("lnk2", "assets/css/lnk2"));
            feo.files = feo.files.sort((a, b) => (a.inputPath.localeCompare(b.inputPath)));
            filesExpected = filesExpected.sort((a, b) => (a.inputPath.localeCompare(b.inputPath)));
            assert_1.default.deepEqual(feo.files, filesExpected);
        });
    });
    describe("Negative Tests", () => {
        it("extracts files containing corrupt links", (done) => {
            fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
            let objects = [];
            // copy to itself, should be ignored
            objects.push(new FileMoveObject_1.FileMoveObject(path_1.default.join(pathToTestAssets, `resultFiles/testTemplateExample.pdf`), path_1.default.join(`resultFiles/testTemplateExample.pdf`)));
            // not existent
            objects.push(new FileMoveObject_1.FileMoveObject(path_1.default.join(pathToTestAssets, `notexistent.in`), path_1.default.join("export", "notexistent.out")));
            // invalid file
            objects.push(new FileMoveObject_1.FileMoveObject(path_1.default.join(pathToTestAssets, `notexistent.in`), undefined));
            main_1.FileExtractor.extractAll(objects, __dirname, path_1.default.join(__dirname, pathToTestAssets), 15000).then(() => {
                try {
                    assert_1.default.ok(fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExample.pdf`)));
                    assert_1.default.ok(!fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, `notexistent.in`)));
                    assert_1.default.ok(!fs.existsSync(path_1.default.join(__dirname, `resultFiles`, "export", "notexistent.out")));
                }
                catch (err) {
                    return done(err);
                }
                done();
            }, (err) => {
                done(err);
            });
        }).timeout(20000);
    });
});
