"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const rimraf_1 = __importDefault(require("rimraf"));
const main_1 = require("../../main");
const PromiseCounter_1 = require("../../util/PromiseCounter");
const AssertExtensions_1 = __importDefault(require("../helpers/AssertExtensions"));
const pathToTestAssets = '../../../testAssets/';
const pathToAssets = '../../../assets/elearnjs/';
const exampleMarkdown = `# This is simple example markdown.

We will insert an image here ![Image](withSomeLink.jpg).

## Heading 2

### Heading 3`;
const exampleMeta = `<!--meta
    Title: Test
    Custom: "<script src='someSource.js'></script>"
-->`;
const exampleImprint = `<!--imprint
    #### elearn.js Template
    <!-- not hidden comment -->
    Universität Hamburg
-->`;
const exampleMeta2 = `---
    Title: Test
    Custom: "<script src='someSource.js'></script>"
---`;
const exampleImprint2 = `\`\`\`imprint
    #### elearn.js Template
    <!-- not hidden comment -->
    Universität Hamburg
\`\`\``;
describe('HTML Converter Setup', () => {
    it('creates an HtmlConverter with default settings', () => {
        let conv = new main_1.HtmlConverter();
        // assert defaults are set
        assert_1.default.equal(conv.getOption("newSectionOnHeading"), true);
        assert_1.default.equal(conv.getOption("headingDepth"), 3);
        assert_1.default.equal(conv.getOption("useSubSections"), true);
        assert_1.default.equal(conv.getOption("subSectionLevel"), 3);
        assert_1.default.equal(conv.getOption("subsubSectionLevel"), 4);
    });
    it('creates an HtmlConverter with correct settings', () => {
        let conv = new main_1.HtmlConverter({
            headingDepth: 2,
            newSectionOnHeading: false,
            useSubSections: false,
            // keep default subSectionLevel
            subsubSectionLevel: 5,
        });
        assert_1.default.equal(conv.getOption("headingDepth"), 2);
        assert_1.default.equal(conv.getOption("newSectionOnHeading"), false);
        assert_1.default.equal(conv.getOption("useSubSections"), false);
        assert_1.default.equal(conv.getOption("subSectionLevel"), 3);
        assert_1.default.equal(conv.getOption("subsubSectionLevel"), 5);
    });
    it('updates settings in HtmlConverter', () => {
        let conv = new main_1.HtmlConverter();
        assert_1.default.notEqual(conv.getOption("headingDepth"), 2);
        assert_1.default.notEqual(conv.getOption("newSectionOnHeading"), false);
        assert_1.default.notEqual(conv.getOption("useSubSections"), false);
        assert_1.default.notEqual(conv.getOption("subSectionLevel"), 4);
        assert_1.default.notEqual(conv.getOption("subsubSectionLevel"), 5);
        conv.setOptions({
            headingDepth: 2,
            newSectionOnHeading: false,
            useSubSections: false,
            subSectionLevel: 4,
            subsubSectionLevel: 5,
        });
        assert_1.default.equal(conv.getOption("headingDepth"), 2);
        assert_1.default.equal(conv.getOption("newSectionOnHeading"), false);
        assert_1.default.equal(conv.getOption("useSubSections"), false);
        assert_1.default.equal(conv.getOption("subSectionLevel"), 4);
        assert_1.default.equal(conv.getOption("subsubSectionLevel"), 5);
        conv.setOptions({
            subSectionLevel: 3,
        });
        assert_1.default.equal(conv.getOption("headingDepth"), 2);
        assert_1.default.equal(conv.getOption("newSectionOnHeading"), false);
        assert_1.default.equal(conv.getOption("useSubSections"), false);
        assert_1.default.equal(conv.getOption("subSectionLevel"), 3);
        assert_1.default.equal(conv.getOption("subsubSectionLevel"), 5);
    });
    it('creates correct subsections', () => __awaiter(this, void 0, void 0, function* () {
        let htmlConverter = new main_1.HtmlConverter({
            subSectionLevel: 2,
            subsubSectionLevel: 3,
        });
        let html = htmlConverter.toHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown, { bodyOnly: true });
        let text = yield html;
        AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToHtmlBodySubSub.html`));
    }));
});
describe('HTML conversion', () => {
    let htmlConverter;
    before(() => {
        htmlConverter = new main_1.HtmlConverter();
    });
    afterEach(() => __awaiter(this, void 0, void 0, function* () {
        let promises = [];
        // remove all folders that might have been created
        let foldersToRemove = ["export"];
        foldersToRemove.forEach((folder) => {
            let promise = new Promise((res, rej) => {
                rimraf_1.default(path_1.default.join(__dirname, pathToTestAssets, folder), (err) => {
                    if (err)
                        rej(err);
                    else
                        res();
                });
            });
            promises.push(promise);
        });
        yield new PromiseCounter_1.PromiseCounter(promises, 15000);
    }));
    describe('With small example codes', () => {
        // basic body only test
        it('creates a valid html body', () => __awaiter(this, void 0, void 0, function* () {
            let html = htmlConverter.toHtml(exampleMarkdown, { bodyOnly: true });
            let text = yield html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToHtmlBody.html`));
        }));
        // basic body only test
        it('creates a valid html body with meta and imprint left out', () => __awaiter(this, void 0, void 0, function* () {
            let html = htmlConverter.toHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown, { bodyOnly: true });
            let text = yield html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToHtmlBody.html`));
        }));
        // basic full document test
        it('creates a valid html document', () => __awaiter(this, void 0, void 0, function* () {
            let html = htmlConverter.toHtml(exampleMarkdown);
            let text = yield html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToHtmlFull.html`));
        }));
        // basic full document test
        it('creates a valid html document with meta and imprint', () => __awaiter(this, void 0, void 0, function* () {
            let html = htmlConverter.toHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown);
            let text = yield html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToHtmlFullImprintMeta.html`));
        }));
        // basic full document test
        it('creates a valid html document with meta and imprint (different syntax)', () => __awaiter(this, void 0, void 0, function* () {
            let html = htmlConverter.toHtml(exampleMeta2 + "\n" + exampleImprint2 + "\n" + exampleMarkdown);
            let text = yield html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToHtmlFullImprintMeta.html`));
        }));
        // basic full document with export ready links
        it('creates a valid html doc with updated file links', () => __awaiter(this, void 0, void 0, function* () {
            let text = yield htmlConverter.toHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown, {
                automaticExtensionDetection: true,
                includeQuiz: false,
                includeElearnVideo: false,
                includeClickImage: false,
                includeTimeSlider: false,
            });
            text = main_1.FileExtractor.replaceAllLinks(text).html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToHtmlFullExtractFiles.html`));
        }));
    });
    describe('with the template', () => {
        it('creates the correct document without extensions', () => __awaiter(this, void 0, void 0, function* () {
            let data = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), 'utf8');
            let text = yield htmlConverter.toHtml(data, { language: "de" });
            text = main_1.FileExtractor.replaceAllLinks(text).html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExample.html`));
        }));
        it('creates the correct document with comment removal', () => __awaiter(this, void 0, void 0, function* () {
            let data = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), 'utf8');
            let text = yield htmlConverter.toHtml(data, {
                language: "de",
                removeComments: true,
            });
            text = main_1.FileExtractor.replaceAllLinks(text).html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExampleNoComments.html`));
        }));
        it('creates the correct document with extensions', () => __awaiter(this, void 0, void 0, function* () {
            let data = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), 'utf8');
            let text = yield htmlConverter.toHtml(data, {
                language: "de",
                includeQuiz: true,
                includeElearnVideo: true,
                includeClickImage: true,
                includeTimeSlider: true,
            });
            text = main_1.FileExtractor.replaceAllLinks(text).html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExampleExtensions.html`));
        }));
        it('creates the correct document with extension detection', () => __awaiter(this, void 0, void 0, function* () {
            let data = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), 'utf8');
            let text = yield htmlConverter.toHtml(data, {
                language: "de",
                automaticExtensionDetection: true,
            });
            text = main_1.FileExtractor.replaceAllLinks(text).html;
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExampleExtensions.html`));
        }));
        it('creates the correct file with extension export', () => __awaiter(this, void 0, void 0, function* () {
            let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
            let data = inBuf.toString();
            // create output folder
            fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
            // convert the file
            yield htmlConverter.toFile(data, path_1.default.join(__dirname, pathToTestAssets, "export", "out.html"), path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                language: "de",
                automaticExtensionDetection: true,
                exportAssets: true,
            });
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "out.html"), path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExampleToFile.html`));
            // check for all the js files, assume everything worked then
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "elearn.js"), path_1.default.join(__dirname, pathToAssets, "assets", "js", "elearn.js"));
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "quiz.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "quiz", "assets", "js", "quiz.js"));
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "elearnvideo.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "elearnvideo", "assets", "js", "elearnvideo.js"));
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "clickimage.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "clickimage", "assets", "js", "clickimage.js"));
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js", "timeslider.js"), path_1.default.join(__dirname, pathToAssets, "extensions", "timeslider", "assets", "js", "timeslider.js"));
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "font", "eLearn-Icons.woff"), path_1.default.join(__dirname, pathToAssets, "assets", "font", "eLearn-Icons.woff"));
        })).slow(500);
        it('creates the correct file with linked file export', () => __awaiter(this, void 0, void 0, function* () {
            let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
            let data = inBuf.toString();
            // create output folder
            fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
            // convert the file
            yield htmlConverter.toFile(data, path_1.default.join(__dirname, pathToTestAssets, "export", "out.html"), path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                language: "de",
                automaticExtensionDetection: true,
                exportLinkedFiles: true,
            });
            // check the output result
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "out.html"), path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExampleExtensions.html`));
            // check for all the js files, assume everything worked then
            AssertExtensions_1.default.assertFilesEqual(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "img", "illu-concept.png"), path_1.default.join(__dirname, pathToTestAssets, "inputFiles", "images", "illu-concept.png"));
            // no exported assets
            assert_1.default.ok(!fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", "assets", "js")));
        })).slow(250);
        it('does not create the file, no path given', () => __awaiter(this, void 0, void 0, function* () {
            let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
            let data = inBuf.toString();
            // create output folder
            fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
            // convert the file
            yield assert_1.default.rejects(htmlConverter.toFile(data, undefined, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                language: "de",
                automaticExtensionDetection: true,
                renderDelay: 5000,
            }));
        })).slow(20000).timeout(30000);
        it('does not create the file, file exists already', () => __awaiter(this, void 0, void 0, function* () {
            let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
            let data = inBuf.toString();
            // create output folder
            fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
            // create dummy
            let file = fs.openSync(path_1.default.join(__dirname, pathToTestAssets, "export", "dummy"), "w+");
            fs.closeSync(file);
            // convert the file
            yield assert_1.default.rejects(htmlConverter.toFile(data, path_1.default.join(__dirname, pathToTestAssets, "export", "dummy"), path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                language: "de",
                automaticExtensionDetection: true,
            }));
        })).slow(20000).timeout(30000);
        it('creates the file, overwrite existing file', () => __awaiter(this, void 0, void 0, function* () {
            let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
            let data = inBuf.toString();
            // create output folder
            fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
            // create dummy
            let file = fs.openSync(path_1.default.join(__dirname, pathToTestAssets, "export", "dummy"), "w+");
            fs.closeSync(file);
            // convert the file
            yield assert_1.default.doesNotReject(htmlConverter.toFile(data, path_1.default.join(__dirname, pathToTestAssets, "export", "dummy"), path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                language: "de",
                automaticExtensionDetection: true,
            }, true));
        })).slow(20000).timeout(30000);
    });
});
