"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const puppeteer_1 = __importDefault(require("puppeteer"));
const rimraf_1 = __importDefault(require("rimraf"));
const main_1 = require("../../main");
const PromiseCounter_1 = require("../../util/PromiseCounter");
const AssertExtensions_1 = __importDefault(require("../helpers/AssertExtensions"));
const PostProcessing_1 = __importDefault(require("../helpers/PostProcessing"));
const pathToTestAssets = '../../../testAssets/';
const pathToAssets = '../../../assets/elearnjs/';
const exampleMarkdown = `# This is simple example markdown.

We will insert an image here ![Image](withSomeLink.jpg).`;
const exampleMeta = `<!--meta
    Title: Test
    Custom: "<script src='someSource.js'></script>"
-->`;
const exampleImprint = `<!--imprint
    #### elearn.js Template
    Universität Hamburg
-->`;
describe('PDF Converter Setup', () => {
    it('creates a PdfConverter with default settings', () => {
        let converter = new main_1.PdfConverter();
        // assert defaults are set
        assert_1.default.equal(converter.getOption("newSectionOnHeading"), true);
        assert_1.default.equal(converter.getOption("headingDepth"), 3);
        assert_1.default.equal(converter.getOption("useSubSections"), true);
        assert_1.default.equal(converter.getOption("subSectionLevel"), 3);
        assert_1.default.equal(converter.getOption("subsubSectionLevel"), 4);
        assert_1.default.equal(converter.getOption("newPageOnSection"), true);
        assert_1.default.equal(converter.getOption("contentZoom"), 1);
        assert_1.default.equal(converter.getOption("customHeader"), undefined);
        assert_1.default.equal(converter.getOption("customFooter"), undefined);
        assert_1.default.equal(converter.getOption("headerHeight"), "0");
        assert_1.default.equal(converter.getOption("footerHeight"), "17mm");
        assert_1.default.equal(converter.getOption("customStyleFile"), undefined);
        assert_1.default.equal(converter.getOption("chromePath"), undefined);
        assert_1.default.equal(converter.getOption("puppeteerOptions"), undefined);
        assert_1.default.equal(converter.getOption("keepChromeAlive"), false);
    });
    it('creates an PdfConverter with correct settings', () => {
        let converter = new main_1.PdfConverter({
            headingDepth: 2,
            newSectionOnHeading: false,
            useSubSections: false,
            subSectionLevel: 4,
            subsubSectionLevel: 5,
            contentZoom: 2,
            customFooter: "testfooter",
            customHeader: "testheader",
            // keep default footer
            headerHeight: "20mm",
            customStyleFile: "somefile",
            newPageOnSection: false,
        });
        assert_1.default.equal(converter.getOption("headingDepth"), 2);
        assert_1.default.equal(converter.getOption("newSectionOnHeading"), false);
        assert_1.default.equal(converter.getOption("useSubSections"), false);
        assert_1.default.equal(converter.getOption("subSectionLevel"), 4);
        assert_1.default.equal(converter.getOption("subsubSectionLevel"), 5);
        assert_1.default.equal(converter.getOption("contentZoom"), 2);
        assert_1.default.equal(converter.getOption("customFooter"), "testfooter");
        assert_1.default.equal(converter.getOption("customHeader"), "testheader");
        assert_1.default.equal(converter.getOption("footerHeight"), "17mm");
        assert_1.default.equal(converter.getOption("headerHeight"), "20mm");
        assert_1.default.equal(converter.getOption("customStyleFile"), "somefile");
        assert_1.default.equal(converter.getOption("newPageOnSection"), false);
        assert_1.default.equal(converter.getOption("chromePath"), undefined);
        assert_1.default.equal(converter.getOption("puppeteerOptions"), undefined);
        assert_1.default.equal(converter.getOption("keepChromeAlive"), false);
    });
    it('should update settings in PdfConverter', () => {
        let conv = new main_1.PdfConverter();
        conv.setOptions({
            headingDepth: 2,
            newSectionOnHeading: false,
            useSubSections: false,
            subSectionLevel: 4,
            subsubSectionLevel: 5,
            contentZoom: 2,
            customFooter: "testfooter",
            customHeader: "testheader",
            footerHeight: "20mm",
            headerHeight: "20mm",
            customStyleFile: "somefile",
            newPageOnSection: false,
        });
        assert_1.default.equal(conv.getOption("headingDepth"), 2);
        assert_1.default.equal(conv.getOption("newSectionOnHeading"), false);
        assert_1.default.equal(conv.getOption("useSubSections"), false);
        assert_1.default.equal(conv.getOption("subSectionLevel"), 4);
        assert_1.default.equal(conv.getOption("subsubSectionLevel"), 5);
        assert_1.default.equal(conv.getOption("contentZoom"), 2);
        assert_1.default.equal(conv.getOption("customFooter"), "testfooter");
        assert_1.default.equal(conv.getOption("customHeader"), "testheader");
        assert_1.default.equal(conv.getOption("footerHeight"), "20mm");
        assert_1.default.equal(conv.getOption("headerHeight"), "20mm");
        assert_1.default.equal(conv.getOption("customStyleFile"), "somefile");
        assert_1.default.equal(conv.getOption("newPageOnSection"), false);
        // update only one setting
        conv.setOptions({
            contentZoom: 1.5,
        });
        assert_1.default.equal(conv.getOption("headingDepth"), 2);
        assert_1.default.equal(conv.getOption("newSectionOnHeading"), false);
        assert_1.default.equal(conv.getOption("useSubSections"), false);
        assert_1.default.equal(conv.getOption("subSectionLevel"), 4);
        assert_1.default.equal(conv.getOption("subsubSectionLevel"), 5);
        assert_1.default.equal(conv.getOption("contentZoom"), 1.5);
        assert_1.default.equal(conv.getOption("customFooter"), "testfooter");
        assert_1.default.equal(conv.getOption("customHeader"), "testheader");
        assert_1.default.equal(conv.getOption("footerHeight"), "20mm");
        assert_1.default.equal(conv.getOption("headerHeight"), "20mm");
        assert_1.default.equal(conv.getOption("customStyleFile"), "somefile");
        assert_1.default.equal(conv.getOption("newPageOnSection"), false);
    });
    it('inserts the correct settings', (done) => {
        let footer = `THIS IS THE TEST FOOTER`;
        let header = `THIS IS THE TEST HEADER`;
        let pdfConverter = new main_1.PdfConverter({
            customFooter: footer,
            customHeader: header,
            customStyleFile: path_1.default.join(__dirname, pathToAssets, "extensions", "quiz", "assets", "css", "quiz.css").replace(/\\/, "/"),
        });
        let html = pdfConverter.toHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown);
        html.then((text) => {
            try {
                assert_1.default.ok(text.indexOf('quiz.css') >= 0);
            }
            catch (err) {
                done(err);
                return;
            }
            done();
        }, (err) => {
            done(err);
        });
    });
});
describe('PDF conversion', () => {
    let pdfConverterKeptAlive;
    before(() => {
        pdfConverterKeptAlive = new main_1.PdfConverter({
            keepChromeAlive: true,
        });
    });
    after(() => {
        // close chromium
        pdfConverterKeptAlive.setOption("keepChromeAlive", false);
    });
    afterEach(() => __awaiter(this, void 0, void 0, function* () {
        let promises = [];
        // remove all folders that might have been created
        let foldersToRemove = ["export"];
        foldersToRemove.forEach((folder) => {
            let promise = new Promise((res, rej) => {
                rimraf_1.default(path_1.default.join(__dirname, pathToTestAssets, folder), (err) => {
                    if (err)
                        rej(err);
                    else
                        res();
                });
            });
            promises.push(promise);
        });
        yield new PromiseCounter_1.PromiseCounter(promises, 15000);
    }));
    describe('With small example codes', () => {
        // basic body only test
        it('creates a valid pdf body', () => __awaiter(this, void 0, void 0, function* () {
            let html = pdfConverterKeptAlive.toHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown, { bodyOnly: true });
            let text = yield html;
            text = PostProcessing_1.default.removeAbsolutePaths(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles`));
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToPdfBody.html`));
        }));
        // basic body only test with toPdfHtml
        it('creates a valid pdf body with toPdfHtml', () => __awaiter(this, void 0, void 0, function* () {
            let html = pdfConverterKeptAlive.toPdfHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown, { bodyOnly: true });
            let text = yield html;
            text = PostProcessing_1.default.removeAbsolutePaths(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles`));
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToPdfBody.html`));
        }));
        // basic full document test
        it('creates a valid pdf document without auto detection', () => __awaiter(this, void 0, void 0, function* () {
            let html = pdfConverterKeptAlive.toHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown, {
                language: "de",
            });
            let text = yield html;
            text = PostProcessing_1.default.removeAbsolutePaths(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles`));
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToPdfFull.html`));
        }));
        // basic full document test
        it('creates a valid pdf document', () => __awaiter(this, void 0, void 0, function* () {
            let html = pdfConverterKeptAlive.toHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown, {
                automaticExtensionDetection: true,
                language: "de",
                includeQuiz: false,
                includeElearnVideo: false,
                includeClickImage: false,
                includeTimeSlider: false,
            });
            let text = yield html;
            text = PostProcessing_1.default.removeAbsolutePaths(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles`));
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToPdfFull.html`));
        }));
        // basic full document test with english language selection
        it('creates a valid pdf document with english language', () => __awaiter(this, void 0, void 0, function* () {
            let html = pdfConverterKeptAlive.toHtml(exampleMeta + "\n" + exampleImprint + "\n" + exampleMarkdown, {
                automaticExtensionDetection: true,
                language: "en",
                includeQuiz: false,
                includeElearnVideo: false,
                includeClickImage: false,
                includeTimeSlider: false,
            });
            let text = yield html;
            text = PostProcessing_1.default.removeAbsolutePaths(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles`));
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testToPdfFullEnglish.html`));
        }));
        it('creates the correct document with extensions from template', () => __awaiter(this, void 0, void 0, function* () {
            let data = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), 'utf8');
            let text = yield pdfConverterKeptAlive.toHtml(data, {
                language: "de",
                automaticExtensionDetection: true,
            });
            text = PostProcessing_1.default.removeAbsolutePaths(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles`));
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExamplePdf.html`));
        }));
        it('creates the correct document without comments from template', () => __awaiter(this, void 0, void 0, function* () {
            let data = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), 'utf8');
            let text = yield pdfConverterKeptAlive.toHtml(data, {
                language: "de",
                removeComments: true,
            });
            text = PostProcessing_1.default.removeAbsolutePaths(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles`));
            AssertExtensions_1.default.assertTextFileEqual(text, path_1.default.join(__dirname, pathToTestAssets, `resultFiles/testTemplateExamplePdfNoComments.html`));
        }));
    });
    describe('with puppeteer', () => {
        let puppeteerAvailable = false;
        let puppeteerOptions;
        before(function () {
            return __awaiter(this, void 0, void 0, function* () {
                this.slow(120000);
                this.timeout(240000);
                let optionList = [
                    {},
                    { executablePath: "chrome" },
                    { executablePath: "chromium-browser" },
                ];
                let selectedOptions;
                // check for all predefined option lists if one of it is possible
                // if so, set pdfConverter accordingly and puppeteerAvailable to true
                for (let options of optionList) {
                    try {
                        let browser = yield puppeteer_1.default.launch(options);
                        browser.close();
                        selectedOptions = options;
                        break;
                    }
                    catch (err) {
                        // ignore errors
                    }
                }
                if (selectedOptions) {
                    puppeteerAvailable = true;
                    puppeteerOptions = selectedOptions;
                }
                if (puppeteerAvailable) {
                    console.log("Found available puppeteer config:", selectedOptions);
                }
            });
        });
        beforeEach(() => {
            // reset default options before every test
            if (puppeteerAvailable) {
                pdfConverterKeptAlive.setOption("puppeteerOptions", puppeteerOptions);
            }
        });
        it('restarts puppeteer correctly', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let corruptConfig = {
                    executablePath: "-//notexistent",
                };
                // set tested working config
                pdfConverterKeptAlive.setOption("puppeteerOptions", puppeteerOptions);
                yield assert_1.default.doesNotReject(pdfConverterKeptAlive.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`)));
                assert_1.default.ok(pdfConverterKeptAlive.hasBrowserInstance());
                // try definitely wrong config
                pdfConverterKeptAlive.setOption("puppeteerOptions", corruptConfig);
                // assert global instance is not set anymore
                assert_1.default.ok(!pdfConverterKeptAlive.hasBrowserInstance());
                yield assert_1.default.rejects(pdfConverterKeptAlive.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`)));
                // try tested working config again, check reset
                pdfConverterKeptAlive.setOption("puppeteerOptions", puppeteerOptions);
                yield assert_1.default.doesNotReject(pdfConverterKeptAlive.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`)));
                assert_1.default.ok(pdfConverterKeptAlive.hasBrowserInstance());
            });
        }).slow(40000).timeout(60000);
        it('handles multiple parallel calls correctly', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let newPdfConverter = new main_1.PdfConverter({
                    puppeteerOptions,
                    keepChromeAlive: true,
                });
                assert_1.default.ok(!newPdfConverter.hasBrowserInstance());
                // multiple starts should use one global instances
                // might corrupt if global instance is not used correctly
                let p1 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                let p2 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                let p3 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                yield assert_1.default.doesNotReject(p1);
                yield assert_1.default.doesNotReject(p2);
                yield assert_1.default.doesNotReject(p3);
                // assert one global instance is set.
                assert_1.default.ok(newPdfConverter.hasBrowserInstance());
                // multiple starts should use global instance
                // might corrupt if global instance is not used correctly
                p1 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                p2 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                p3 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                yield assert_1.default.doesNotReject(p1);
                yield assert_1.default.doesNotReject(p2);
                yield assert_1.default.doesNotReject(p3);
                assert_1.default.ok(newPdfConverter.hasBrowserInstance());
                // close instance
                newPdfConverter.setOption("keepChromeAlive", false);
                assert_1.default.ok(!newPdfConverter.hasBrowserInstance());
            });
        }).slow(40000).timeout(60000);
        it('handles parallel keepChromeAlive reset correctly', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let newPdfConverter = new main_1.PdfConverter({
                    puppeteerOptions,
                    keepChromeAlive: true,
                });
                // multiple starts should create multiple local instances
                // might corrupt if global instance is not used correctly
                yield newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                // assert one global instance is set.
                assert_1.default.ok(newPdfConverter.hasBrowserInstance());
                // multiple starts should use global instance
                // might corrupt if global instance is not used correctly
                let p1 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), { renderDelay: 2000 });
                let p2 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), { renderDelay: 2000 });
                let p3 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), { renderDelay: 2000 });
                // wait until processes are most likely started
                // should be deactivated while the render delay is active
                setTimeout(() => {
                    newPdfConverter.setOption("keepChromeAlive", false);
                    // browser seems to have been closed already
                    assert_1.default.ok(!newPdfConverter.hasBrowserInstance());
                }, 1000);
                yield assert_1.default.doesNotReject(p1);
                yield assert_1.default.doesNotReject(p2);
                yield assert_1.default.doesNotReject(p3);
                assert_1.default.ok(!newPdfConverter.hasBrowserInstance());
            });
        }).slow(40000).timeout(60000);
        it('restarts after parallel `puppeteerOptions` change', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let corruptConfig = {
                    executablePath: "-//notexistent",
                };
                // set tested working config
                pdfConverterKeptAlive.setOption("puppeteerOptions", puppeteerOptions);
                yield assert_1.default.doesNotReject(pdfConverterKeptAlive.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`)));
                // assert global instance set
                assert_1.default.ok(pdfConverterKeptAlive.hasBrowserInstance());
                // start conversion 1
                let p1 = pdfConverterKeptAlive.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), { renderDelay: 5000 });
                // wait one second
                yield new Promise((res, rej) => {
                    setTimeout(res, 1000);
                });
                // reset config to corrupt one
                pdfConverterKeptAlive.setOption("puppeteerOptions", corruptConfig);
                // browser seems to have been closed already
                assert_1.default.ok(!pdfConverterKeptAlive.hasBrowserInstance());
                // try definitely wrong config, before p1 has resolved
                // rejects only if restarted correctly
                let p2 = pdfConverterKeptAlive.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                yield assert_1.default.rejects(p2);
                yield assert_1.default.doesNotReject(p1);
                // instance should be closed now, due to corrupt restart of p2
                assert_1.default.ok(!pdfConverterKeptAlive.hasBrowserInstance());
            });
        }).slow(40000).timeout(60000);
        it('creates multiple local instances without `keepChromeAlive`', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let newPdfConverter = new main_1.PdfConverter({
                    puppeteerOptions,
                });
                assert_1.default.ok(!newPdfConverter.hasBrowserInstance());
                // multiple starts should use one global instances
                // might corrupt if global instance is not used correctly
                let p1 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                let p2 = newPdfConverter.toBuffer(exampleMarkdown, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`));
                assert_1.default.ok(!newPdfConverter.hasBrowserInstance());
                yield assert_1.default.doesNotReject(p1);
                yield assert_1.default.doesNotReject(p2);
                assert_1.default.ok(!newPdfConverter.hasBrowserInstance());
            });
        }).slow(40000).timeout(60000);
        it('creates the correct file', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
                let data = inBuf.toString();
                // create output folder
                fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
                // set options
                pdfConverterKeptAlive.setOptions({
                    headerHeight: "0cm",
                });
                // convert the file
                yield pdfConverterKeptAlive.toFile(data, path_1.default.join(__dirname, pathToTestAssets, "export", "out.pdf"), path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                    language: "de",
                    automaticExtensionDetection: true,
                    renderDelay: 5000,
                });
                assert_1.default.ok(fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", "out.pdf")));
            });
        }).slow(40000).timeout(60000);
        it('creates the correct file with delay', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
                let data = inBuf.toString();
                // create output folder
                fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
                // set options
                pdfConverterKeptAlive.setOptions({
                    headerHeight: "0cm",
                });
                let startTime = new Date();
                // convert the file
                yield pdfConverterKeptAlive.toFile(data, path_1.default.join(__dirname, pathToTestAssets, "export", "out.pdf"), path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                    language: "de",
                    automaticExtensionDetection: true,
                    renderDelay: 10000,
                });
                let endTime = new Date();
                assert_1.default.ok(endTime.getTime() - startTime.getTime() >= 10000);
                assert_1.default.ok(fs.existsSync(path_1.default.join(__dirname, pathToTestAssets, "export", "out.pdf")));
            });
        }).slow(40000).timeout(60000);
        it('should not create the file, no path given', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
                let data = inBuf.toString();
                // create output folder
                fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
                // convert the file
                yield assert_1.default.rejects(pdfConverterKeptAlive.toFile(data, undefined, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                    language: "de",
                    automaticExtensionDetection: true,
                    renderDelay: 5000,
                }));
            });
        }).slow(40000).timeout(60000);
        it('should not create the file, file exists already', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
                let data = inBuf.toString();
                // create output folder
                fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
                // create dummy
                let file = fs.openSync(path_1.default.join(__dirname, pathToTestAssets, "export", "dummy"), "w+");
                fs.closeSync(file);
                // convert the file
                yield assert_1.default.rejects(pdfConverterKeptAlive.toFile(data, path_1.default.join(__dirname, pathToTestAssets, "export", "dummy"), path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                    language: "de",
                    automaticExtensionDetection: true,
                }));
            });
        }).slow(40000).timeout(60000);
        it('creates the file, overwrite existing file', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
                let data = inBuf.toString();
                // create output folder
                fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
                // create dummy
                let file = fs.openSync(path_1.default.join(__dirname, pathToTestAssets, "export", "dummy"), "w+");
                fs.closeSync(file);
                // convert the file
                yield assert_1.default.doesNotReject(pdfConverterKeptAlive.toFile(data, path_1.default.join(__dirname, pathToTestAssets, "export", "dummy"), path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                    language: "de",
                    automaticExtensionDetection: true,
                }, true));
            });
        }).slow(40000).timeout(60000);
        it('creates a buffer', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
                let data = inBuf.toString();
                // create output folder
                fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
                // set some options
                pdfConverterKeptAlive.setOptions({
                    headerHeight: "2in",
                    footerHeight: "10px",
                });
                // convert the file
                yield pdfConverterKeptAlive.toBuffer(data, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                    language: "de",
                    automaticExtensionDetection: true,
                });
            });
        }).slow(40000).timeout(60000);
        it('creates a buffer with incorrect footerHeight', function () {
            return __awaiter(this, void 0, void 0, function* () {
                if (!puppeteerAvailable) {
                    console.log("Puppeteer is not available on this device. Skipping this test.");
                    this.skip();
                }
                let inBuf = fs.readFileSync(path_1.default.join(__dirname, pathToTestAssets, `inputFiles/testTemplateExample.md`), { encoding: 'utf8' });
                let data = inBuf.toString();
                // create output folder
                fs.mkdirSync(path_1.default.join(__dirname, pathToTestAssets, "export"));
                // set some options
                pdfConverterKeptAlive.setOptions({
                    headerHeight: undefined,
                    footerHeight: "10pxls",
                });
                // convert the file
                yield pdfConverterKeptAlive.toBuffer(data, path_1.default.join(__dirname, pathToTestAssets, `inputFiles`), {
                    language: "de",
                    automaticExtensionDetection: true,
                });
            });
        }).slow(40000).timeout(60000);
    });
});
